# Hurry Curry! - a game about cooking
# Copyright (C) 2025 Hurry Curry! contributors
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License only.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
extends BlurSetup
class_name ServerMessage

const DEFAULT_FONT = preload("res://gui/resources/fonts/font-josefin-sans.woff2")
const MONOSPACE_FONT = preload("res://gui/resources/fonts/font-azaret-mono.woff2")

@onready var label: Label = $CenterContainer/Label

func set_text(text: String, use_monospace := true):
	label.text = text
	var font: FontVariation = label.get_theme_font("font")
	font.base_font = MONOSPACE_FONT if use_monospace else DEFAULT_FONT
	label.add_theme_font_size_override("font_size", 16 if use_monospace else 20)
