# Hurry Curry! - a game about cooking
# Copyright (C) 2025 Hurry Curry! contributors
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License only.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
class_name CounterBase
extends Counter

func _init(ctx: TileFactory.TileCC):
	super(ctx)
	match kind:
		CounterKind.OUTER_CORNER:
			base.add_child(load("res://map/tiles/counter_outer_corner.tscn").instantiate())
		CounterKind.STRAIGHT:
			base.add_child(load("res://map/tiles/counter_straight.tscn").instantiate())
		CounterKind.STRAIGHT_BACKSPLASH:
			base.add_child(load("res://map/tiles/counter_straight_backsplash.tscn").instantiate())
