# Hurry Curry! - a game about cooking
# Copyright (C) 2025 Hurry Curry! contributors
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License only.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
class_name Entry
extends Menu

func _ready():
	super()
	get_window().title = "Hurry Curry!"

	if not Cli.init(): 
		get_tree().quit()
		return

	if Cli.opts.has("render-items") or Cli.opts.has("render-tiles"):
		await submenu("res://system/render_tool.tscn")
	if Cli.opts.has("connect_address"):
		await submenu("res://gui/menus/game.tscn", Cli.opts["connect_address"])
	elif not Settings.read("gameplay.setup_completed"):
		await submenu("res://gui/menus/setup/setup.tscn")
	else:
		await submenu("res://gui/menus/main/main.tscn")

	print("Menu stack empty, quitting game.")
	get_tree().quit()

func quit():
	pass
