# Hurry Curry! - a game about cooking
# Copyright (C) 2025 Hurry Curry! contributors
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License only.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
extends Control
class_name ChatPreview

const CHAT_MESSAGE_SCENE = preload("res://gui/components/message/chat_message.tscn")

@onready var game: Game = $"../../Game" # TODO
@onready var messages_container: VBoxContainer = $MarginContainer/ScrollContainer/PanelContainer/Messages

func _ready():
	game.text_message.connect(func(message: Game.TextMessage):
		var chat_message: ChatMessage = CHAT_MESSAGE_SCENE.instantiate()
		messages_container.add_child(chat_message)
		chat_message.set_message(
			message.username,
			message.text,
			message.color,
			true,
			message.timeout_remaining)
	)
