# Hurry Curry! - a game about cooking
# Copyright (C) 2025 Hurry Curry! contributors
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License only.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
class_name MessageParser
extends Object

var raw: Dictionary
var kind := Kind.UNKNOWN
var result = null

enum Kind {
	ITEM,
	TILE,
	TEXT,
	TRANSLATION,
	UNKNOWN
}

func _init(raw_: Dictionary) -> void:
	raw = raw_

func parse(game: Game) -> String:
	if "text" in raw:
		kind = Kind.TEXT
		result = raw["text"]
	elif "translation" in raw:
		kind = Kind.TRANSLATION
		# TODO: move function here
		result = Global.get_message_str(raw)
	elif "tile" in raw:
		kind = Kind.TILE
		result = game.tile_names[raw["tile"]]
	elif "item" in raw:
		kind = Kind.ITEM
		result = game.item_names[raw["item"]]
	return result
