# Hurry Curry! - a game about cooking
# Copyright (C) 2025 Hurry Curry! contributors
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License only.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
class_name ButtonSetting
extends GameSetting

var callback

func _init(new_id: String, new_default, callback_):
	callback = callback_
	super (new_id, new_default)

func hide_reset(): return true

func create_row():
	var row = super ()
	row.value_node = Button.new()
	row.value_node.text = tr(nskey + ".button_label")
	row.value_node.pressed.connect(callback)
	return row
