/*
    Hurry Curry! - a game about cooking
    Copyright (C) 2025 Hurry Curry! Contributors

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU Affero General Public License as published by
    the Free Software Foundation, version 3 of the License only.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Affero General Public License for more details.

    You should have received a copy of the GNU Affero General Public License
    along with this program.  If not, see <https://www.gnu.org/licenses/>.

*/
use super::{Entity, EntityContext};
use anyhow::{Result, anyhow};
use hurrycurry_protocol::{PacketC, glam::Vec2};

#[derive(Debug, Default, Clone)]
pub struct PlayerPortal {
    pub(super) from: Vec2,
    pub(super) to: Vec2,
}

impl Entity for PlayerPortal {
    fn tick(&mut self, c: EntityContext) -> Result<()> {
        let mut players = Vec::new();
        c.game
            .players_spatial_index
            .query(self.from, 0.5, |pid, _| players.push(pid));

        for pid in players {
            let p = c
                .game
                .players
                .get_mut(&pid)
                .ok_or(anyhow!("Player is missing"))?;
            p.movement.position = self.to;
            c.packet_out
                .push_back(PacketC::MovementSync { player: pid });
        }

        Ok(())
    }
}
