# Hurry Curry! - a game about cooking
# Copyright (C) 2025 Hurry Curry! contributors
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License only.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
extends CanvasLayer

@onready var boost = $Buttons/Boost
@onready var interact_right = $Buttons/InteractR
@onready var interact_left = $Buttons/InteractL
@onready var pause = $Buttons2/Pause
@onready var modulate_color = $MovementStick.pressed_color

var touch_enabled := false

func _ready():
	Settings.hook_changed_init("ui.touch_controls", self, apply_touch)
	Global.using_touch_change.connect(func f(_x): apply_touch(Settings.read("ui.touch_controls"))) # throw away useless argument
	Global.hand_count_change.connect(apply_hand_count)
	apply_hand_count(Global.hand_count)

func apply_touch(touch_controls: String):
	touch_enabled = Global.using_touch or touch_controls == "enabled" and touch_controls != "disabled"
	visible = touch_enabled

func apply_hand_count(count: int):
	interact_left.visible = count >= 2

func in_lobby_updated(in_lobby: bool):
	$Buttons.visible = not in_lobby

func _on_boost_pressed():
	Input.action_press("boost")
	boost.modulate = modulate_color

func _on_boost_released():
	Input.action_release("boost")
	boost.modulate = Color.WHITE

func _on_interact_r_pressed():
	if Global.hand_count == 1:
		Input.action_press("interact_left")
	else:
		Input.action_press("interact_right")
	interact_right.modulate = modulate_color

func _on_interact_r_released():
	if Global.hand_count == 1:
		Input.action_release("interact_left")
	else:
		Input.action_release("interact_right")
	interact_right.modulate = Color.WHITE

func _on_pause_pressed():
	Input.action_press("ui_menu")
	pause.modulate = modulate_color

func _on_pause_released():
	Input.action_release("ui_menu")
	pause.modulate = Color.WHITE

func _on_interact_l_pressed() -> void:
	Input.action_press("interact_left")
	interact_left.modulate = modulate_color

func _on_interact_l_released() -> void:
	Input.action_release("interact_left")
	interact_left.modulate = Color.WHITE
