/*
    Hurry Curry! - a game about cooking
    Copyright (C) 2025 Hurry Curry! Contributors

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU Affero General Public License as published by
    the Free Software Foundation, version 3 of the License only.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Affero General Public License for more details.

    You should have received a copy of the GNU Affero General Public License
    along with this program.  If not, see <https://www.gnu.org/licenses/>.

*/
#![feature(random)]
pub mod algos;
pub mod pathfinding;

use hurrycurry_client_lib::Game;
use hurrycurry_protocol::{
    PacketS, PlayerID,
    glam::{IVec2, Vec2},
};
use std::random::random;

#[derive(Default, Clone)]
pub struct BotInput {
    pub direction: Vec2,
    pub boost: bool,
    pub interact: Option<IVec2>,
    pub leave: bool,
    pub extra: Vec<PacketS>,
}

pub type DynBotAlgo = Box<dyn BotAlgo + Send + Sync + 'static>;
pub trait BotAlgo {
    fn tick(&mut self, me: PlayerID, game: &Game, dt: f32) -> BotInput;
}

impl<T: BotAlgo + ?Sized> BotAlgo for Box<T> {
    fn tick(&mut self, me: PlayerID, game: &Game, dt: f32) -> BotInput {
        (**self).tick(me, game, dt)
    }
}

fn random_float() -> f32 {
    random::<u32>(..) as f32 / u32::MAX as f32
}
