# Hurry Curry! - a game about cooking
# Copyright (C) 2025 Hurry Curry! contributors
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License only.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
class_name Effect
extends Node3D

@onready var success = $Success
@onready var failure = $Failure
@onready var angry_grunt: PlayRandom = $AngryGrunt
@onready var stars = $Stars
@onready var angry = $Angry


func set_effect(e: String):
	match e:
		"satisfied":
			stars.emitting = true
			success.play()
		"angry":
			angry.emitting = true
			angry_grunt.play_random()
			failure.play()
		_:
			push_warning("effect %s unknown" % e)
