# Hurry Curry! - a game about cooking
# Copyright (C) 2025 Hurry Curry! contributors
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License only.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
extends LightSetup
class_name SkyLightSetup

func _ready():
	super()

func set_sky(sky_name: String):
	match sky_name:
		"day":
			light_color = Color.WHITE
			light_energy = 1.
		"night":
			light_color = Color("427a9a")
			light_energy = 0.4
		_:
			push_error("Can not set up light: Unsupported sky name \"%s\"" % sky_name)
