# Hurry Curry! - a game about cooking
# Copyright (C) 2025 Hurry Curry! contributors
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License only.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
class_name ConveyorModel
extends Node3D

var alpha: float = 0.

@onready var mat: ShaderMaterial = $ConveyorDirection.get_active_material(0)

func _process(delta):
	alpha = G.interpolate(alpha, 0., delta)
	mat.set_shader_parameter("alpha_fac", alpha)

func to_direction(direction: Vector2):
	direction.y *= -1
	$ConveyorDirection.rotation.y = direction.angle() + .5 * PI
	alpha = 1.
