# Hurry Curry! - a game about cooking
# Copyright (C) 2025 Hurry Curry! contributors
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License only.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
class_name ItemBubble
extends MeshInstance3D

var item: Item
var timeout_remaining := 0.
var timeout_initial := 0.
var progress_style = preload("res://gui/resources/style/item_bubble_progress_style.tres")

@onready var item_message = $SubViewport/ItemMessage

func set_item(item_name: String, timeout_initial_: float, timeout_remaining_: float):
	item_message.set_item(item_name, timeout_initial_, timeout_remaining_)
