# Hurry Curry! - a game about cooking
# Copyright (C) 2025 Hurry Curry! contributors
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License only.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
class_name SettingsCategory
extends GameSetting

var settings: Array # Array[GameSetting]

func _init(new_id: String, new_settings: Array):
	super(new_id)
	settings = new_settings

func set_parent(parent: GameSetting):
	super(parent)
	for c in settings:
		c.set_parent(self)

func create_row():
	var row = ScrollContainerCustom.new()
	var options = VBoxContainer.new()
	row.name = tr(nskey)
	row.size_flags_horizontal = Control.SIZE_EXPAND_FILL
	options.size_flags_horizontal = Control.SIZE_EXPAND_FILL
	row.add_child(options)
	
	for r in settings: options.add_child(r.create_row())
	return row

func load(d: Dictionary):
	for c in settings: c.load(d)

func save(d: Dictionary):
	for c in settings: c.save(d)
