/*
    Hurry Curry! - a game about cooking
    Copyright (C) 2025 Hurry Curry! Contributors

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU Affero General Public License as published by
    the Free Software Foundation, version 3 of the License only.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Affero General Public License for more details.

    You should have received a copy of the GNU Affero General Public License
    along with this program.  If not, see <https://www.gnu.org/licenses/>.

*/
body {
    margin: 0px;
    background-color: rgb(74, 74, 74);
    scroll-behavior: smooth;
}

.pagegroup {
    margin-left: auto;
    margin-right: auto;
    display: block;
    width: 64em;
}

.page {
    display: inline-block;
    background-color: antiquewhite;
    box-sizing: border-box;
    padding: 2em;
    margin: 1em;
    aspect-ratio: 0.707;
    width: 30em;
    height: auto;
    overflow-y: auto;
}
