/*
    Hurry Curry! - a game about cooking
    Copyright (C) 2025 Hurry Curry! Contributors

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU Affero General Public License as published by
    the Free Software Foundation, version 3 of the License only.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Affero General Public License for more details.

    You should have received a copy of the GNU Affero General Public License
    along with this program.  If not, see <https://www.gnu.org/licenses/>.

*/
#![feature(if_let_guard, iterator_try_collect, stmt_expr_attributes, random)]
pub mod commands;
pub mod entity;
pub mod interaction;
pub mod network;
pub mod scoreboard;
pub mod server;
pub mod state;

use hurrycurry_protocol::glam::Vec2;
use std::{fmt::Display, random::random};

#[derive(Debug, Clone, Copy, PartialEq, Eq, PartialOrd, Ord, Hash)]
pub struct ConnectionID(pub i64);

impl Display for ConnectionID {
    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
        write!(f, "conn#{}", self.0)
    }
}

pub trait InterpolateExt {
    fn exp_to(&mut self, target: Self, dt: f32);
}
impl InterpolateExt for Vec2 {
    fn exp_to(&mut self, target: Self, dt: f32) {
        self.x = target.x + (self.x - target.x) * (-dt).exp();
        self.y = target.y + (self.y - target.y) * (-dt).exp();
    }
}
impl InterpolateExt for f32 {
    fn exp_to(&mut self, target: Self, dt: f32) {
        *self = target + (*self - target) * (-dt).exp();
    }
}

fn random_float() -> f32 {
    random::<u32>(..) as f32 / u32::MAX as f32
}
fn random_gauss() -> f32 {
    [(); 12].map(|()| random_float()).iter().sum::<f32>() - 6.
}
