# Hurry Curry! - a game about cooking
# Copyright (C) 2025 Hurry Curry! contributors
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License only.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
class_name TextSetting
extends GameSetting

var placeholder: String

func _init(new_id: String, new_default: String, new_placeholder: String = ""):
	super(new_id, new_default)
	placeholder = new_placeholder

func create_row():
	var row = super()
	var input := LineEdit.new()
	input.placeholder_text = placeholder
	input.text_changed.connect(func(text): Settings.write(key, text))
	Settings.hook_changed_init(key, input,
		func(text):
			if is_instance_valid(input):
				var pos = input.caret_column
				input.text = text
				input.caret_column = pos
	)
	row.value_node = input
	return row
